#!/usr/bin/perl

###############################################################################
##OCS inventory-NG Version 1.0 Beta
##Copyleft Pascal DANEK 2005
##Web : http://ocsinventory.sourceforge.net
##
##This code is open source and may be copied and modified as long as the source
##code is always made freely available.
##Please refer to the General Public Licence http://www.gnu.org/ or Licence.txt
################################################################################
#
#Last modifications  : 29/11/2005
#
package Ocsinventory;
use strict;
#To compress the tx and read the rx
use Compress::Zlib;
#To modify if your log path is different
use constant LOGPATH => "/var/log/ocsinventory-NG";
#mod-perl2
############
use Apache2::Connection (); 
use Apache2::SubRequest (); 
use Apache2::Access (); 
use Apache2::RequestIO (); 
use Apache2::RequestUtil (); 
use Apache2::RequestRec (); 
use Apache2::ServerUtil (); 
use Apache2::Log; 
use APR::Table (); 
use Apache2::Const -compile => qw(OK FORBIDDEN SERVER_ERROR HTTP_BAD_REQUEST);
#############
#Declarations 
#############
#The apache object's request
my $r;
#The compressed data
my $data;
#The xml message
my $query;
#The running context(update or new deviceid 
my $update;
#The xml inventory
my $result;
#The DBI object
my $dbh;
#The unique identifier of the device(deviceid)
my $DeviceID;
#An array element to store the fields of account info table
my @accountkeys;
#A simple flag for this functionnalities
my $ipdiscover;
my $registry;
#LogLevel
my $loglevel;
my $red;
my $lost;
######
#MAIN#
##########################################################
#
#The subroutine that will be called by Apache
sub handler{
	#Initialisations
	undef $data;
	undef $query;
 	undef $update;
 	undef $r;
	undef @accountkeys;
	undef $DeviceID;
	undef $result;
	undef $dbh;
	undef $registry;
	undef $ipdiscover;
	undef $red;
	undef $lost;
	
	#LOG FILE
	##########
	#
	#All events will be stored in this file in the csv format(See the errors code in the documentation)
	open LOG, ">>".LOGPATH."/ocsinventory-NG.log" or die "Failed to open log file : $!\n";
	#We don't want buffer, so we allways flush the handles
	select(LOG);
	$|=1;
	select(STDOUT);
	$|=1;
	#Get the data and the apache object
	$r=shift;
	#First, we determine the http method
	#The get method will be only available for the bootstrap to manage the deploy, and maybe, sometime to give files wich will be stored in the database
	if($r->method() eq 'GET'){
		#To manage the first contact with the bootstrap
		#The uri must be "/ocsinventory/deploy/[filename]"
		if($r->uri()=~/deploy\/(.+)\/?$/){
			#Connect to the database
			return Apache2::Const::SERVER_ERROR if(&_database_connect());
			my $request=$dbh->prepare("SELECT IVALUE FROM config WHERE NAME='DEPLOY'");
			$request->execute();
			if($request->rows){
				my $row = $request->fetchrow_hashref;
				unless($row->{'IVALUE'}){
					print LOG localtime().";304;deploy refused - feature is turned off ;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";deploy\n" if $loglevel;
					return Apache2::Const::FORBIDDEN;
				}
			}else{
				print LOG localtime().";503;Cannot get the configuration for deploy;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";deploy\n";
				return Apache2::Const::HTTP_BAD_REQUEST;
			}
			#We want to know if the file is available
			$request=$dbh->prepare("SELECT CONTENT FROM deploy WHERE NAME=".$dbh->quote($1));
			$request->execute();
			#If not, we return a bad request and log the event
			print LOG localtime().";511;File ($1) do not exist;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";deploy\n",return Apache2::Const::HTTP_BAD_REQUEST unless($request->rows);
			#We extract the file and send it
			my $row=$request->fetchrow_hashref();
			#We force this content type to avoid the direct interpretation of, for example, a plain text file
			$r->err_headers_out->{'Cache-control'} = 'max-age=3600';
			$r->err_headers_out->{'Content-length'} = length($row->{"CONTENT"});
			$r->content_type("Application/octet-stream");
			$r->print($row->{"CONTENT"});
			#We log it
			print LOG localtime().";302;File ($1) transmitted;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";deploy\n" if $loglevel;
			return Apache2::Const::OK;
		}elsif($r->uri()=~/update\/(.+)\/(.+)\/(\d+)\/?/){
		#We use the GET method for the update to use the proxies
		#The URL is built like that : [OCSFSERVER]/ocsinventory/[os]/[name]/[version]
		#All the data are writed like tehy are in the database
			my $platform;
			my $request;
			my $row;
			my $name;
			my $version;
			
			#Send file for update
			$platform = $1;
			$name = $2;
			$version = $3;
			unless($platform and $name and $version){
				print LOG localtime().";512;Bad request;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";file\n";
				return Apache2::Const::HTTP_BAD_REQUEST;
			}
			return Apache2::Const::SERVER_ERROR if(&_database_connect());
			$request = $dbh->prepare("SELECT CONTENT FROM files WHERE OS='$platform' AND NAME='$name' AND VERSION='$version'");
			$request->execute();
			unless($request->rows){
				$request->finish;
				print LOG localtime().";511;$name for $platform, version  $version do not exist;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";file\n";
				return Apache2::Const::HTTP_BAD_REQUEST;
			}else{
				$row=$request->fetchrow_hashref();
				#Sending
				$row->{'CONTENT'}=Compress::Zlib::compress($row->{'CONTENT'}) or print LOG localtime().";506;Compression problem;".$ENV{'REMOTE_ADDR'}.";file\n",return Apache2::Const::HTTP_BAD_REQUEST;
				$r->content_type('Application/octet-stream');
				$r->err_headers_out->{'Cache-control'} = 'max-age=3600';
				$r->err_headers_out->{'Content-length'} = length($row->{'CONTENT'});
				$r->print($row->{'CONTENT'});
				print LOG localtime().";200;$name for $platform, version  $version transmitted;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";file\n" if $loglevel;
				$request->finish;
				return Apache2::Const::OK;
			}
		
		}else{
		#If the url is invalid
			return Apache2::Const::HTTP_BAD_REQUEST;
		}
	}
	#Here is the post method management
	elsif($r->method eq 'POST'){
		unless($r->headers_in->{'Content-type'}=~/Application\/x-compress/i){
		#Our discussion is compressed stream, nothing else
			print LOG localtime().";510;Bad content-type :  ".$r->headers_in->{'Content-type'}.";".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";general\n";
			return Apache2::Const::FORBIDDEN;
			
		}
		
		#Get the data
		read(STDIN, $data, $ENV{'CONTENT_LENGTH'});
		#Connect to the database
		return Apache2::Const::SERVER_ERROR if(&_database_connect());
		#Debug level for Apache::DBI (apache/error.log)
		#$Apache::DBI::DEBUG=2;
	
		#Read the request
		#Possibilities :
		#prolog : The agent wants to know if he have to send an inventory(and with wich options)
		#update : The agent wants to know if there is a newer version available
		#inventory : It is an inventory
		#system : Request to know the server's time response (and if it's alive) not yet implemented
		#file : Download files when upgrading (For the moment, only when upgrading)
		##################################################
		#
		#Inflate the data
		my $d=undef;
		my $status=undef;
		#
		$d = Compress::Zlib::inflateInit() or print LOG localtime().";506;Failed to initialise inflation stream;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";general\n", return Apache2::Const::HTTP_BAD_REQUEST;
		($data, $status) = $d->inflate($data);
		unless( $status == Z_OK or $status == Z_STREAM_END){
			print LOG localtime().";506;Inflation problem;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";general\n"; 
			return Apache2::Const::HTTP_BAD_REQUEST;
		}
		##########################
		#Parse the XML request
		unless($query=XML::Simple::XMLin( $data, SuppressEmpty => 1 )){
			warn localtime().";507;XML parsing error : $!".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";general\n";
			return Apache2::Const::HTTP_BAD_REQUEST;
		}
		#Get the request type
		my $request=$query->{QUERY};
		if($request eq 'PROLOG'){
			my $ret = &_prolog_resp();
			return(&_end($ret));
		}elsif($request eq 'UPDATE'){
			my $ret = &_update();
			return(&_end($ret));
		}elsif($request eq 'INVENTORY'){
			my $ret = &_inventory();
			return(&_end($ret))
		}elsif($request eq 'SYSTEM'){
			my $ret = &_system();
			return(&_end($ret));
		}else{
			print LOG localtime().";500;Unauthorised request = $request;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";general\n";
			return Apache2::Const::HTTP_BAD_REQUEST;
		}
	}else{ return Apache2::Const::FORBIDDEN }
	
}
#Database connection
sub _database_connect{
	my $Database;
	my $Port;
	my $Host;
	my $DBuser;
	my $DBpassword;

	#Get the variables declared in httpd.conf
	#Login
	$DBuser=$r->dir_config('DBUSER');
	#Password
	$DBpassword=$r->dir_config('DBPWD');
	#Port
	$Port=$r->dir_config('DBPORT');
	#Host
	$Host=$r->dir_config('DBHOST');
	#
	#To manage A specific database for the non connected computers
	#If no database specified, we take the httpd DBNAME one
	if($r->headers_in->{'User-agent'} =~ /local/i){
	    if($r->dir_config('DBLOCAL')){
	      $Database = $r->dir_config('DBLOCAL');
	    }else{
	      $Database = $r->dir_config('DBNAME');
	    }
	}else{
	    unless($Database = $r->dir_config('DBNAME')){
	      print LOG localtime().";505;No database selected - Abort..;general\n";
	      return(1);
	    }
	}
	#Connection...
	unless($dbh =  DBI->connect("DBI:mysql:database=$Database;host=$Host;port=$Port", $DBuser, $DBpassword)){
		print LOG localtime().";505;Cannot connect to the database - Abort..;general\n";
		return(1);
	}
	0;
}

#Subroutine called at the end of execution
sub _end{
	close(LOG);
	$dbh->disconnect;
	return $_[0];
}

#Proceed with inventory
#######################
#
#Subroutine called for an inventory incoming
sub _inventory{
	#XML to Perl
	unless($result=XML::Simple::XMLin( $data, SuppressEmpty => 1, ForceArray => ['INPUTS', 'CONTROLLERS', 'MEMORIES', 'MONITORS', 'PORTS', 'REGISTRY', 'SOFTWARES', 'STORAGES', 'DRIVES', 'INPUTS', 'MODEMS', 'NETWORKS', 'PRINTERS', 'SLOTS', 'SOUNDS', 'VIDEOS', 'PROCESSES', 'ACCOUNTINFO', 'H'] )){
		print LOG localtime().";507;XML parsing error :< $!;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";inventory\n" ;
		return Apache2::Const::HTTP_BAD_REQUEST;
	}
	
	#Getting the device ID
	$DeviceID=$result->{CONTENT}->{DEVICEID};
	#If we do not find it 
	unless(defined($DeviceID)){
		print LOG localtime().";501;Cannot read deviceID;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";inventory\n";
		return Apache2::Const::HTTP_BAD_REQUEST;
	}
	#If it is no conform
	unless($DeviceID=~/.+-\d{4}-\d{2}-\d{2}-\d{2}-\d{2}-\d{2}/o){
		print LOG localtime().";502;Device ID $DeviceID invalid;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";inventory\n";
		return Apache2::Const::HTTP_BAD_REQUEST;
	}
	#If it is no conform
	unless($result->{CONTENT}->{HARDWARE}->{NAME}=~/\w+/o){
		print LOG localtime().";502;Invalid hostname;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";inventory\n";
		return Apache2::Const::HTTP_BAD_REQUEST;
	}
	#To know more about the situation (update, new machine...)
	&_context();
	#Put the inventory in the database
	#All subroutines below correspond to the tables in the database and sections related in the xml data
	&_hardware(); 
	&_accountinfo() unless $update;
	&_accesslog();
	&_bios();
	&_memories();
	&_slots();
	&_controllers();
	&_monitors();
	&_ports();
	&_storages();
	&_drives();
	&_inputs();
	&_modems();
	&_networks();
	&_printers();
	&_sounds();
	&_videos();
	&_softwares();
	#############
	#
	#Manage several questions, including double entries
	&_postinsert();
	#To get the requested registry keys
	&_registry() if $registry;
	#To get the map of the network, activate the device or deactivate it
	&_ipdiscover() if $ipdiscover;
	#That's all
	print LOG localtime().";101;Inventory of $result->{CONTENT}->{DEVICEID} transmitted;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";inventory\n" if $loglevel;
	return Apache2::Const::OK;		           
}

#We determine the context
sub _context{
	my $request;
	my $row;
	#####
	#Is DeviceID existing in the database ?
	$request = $dbh->prepare("SELECT DEVICEID FROM hardware WHERE DEVICEID=".$dbh->quote($DeviceID));
	$request->execute();
	if($request->rows){
		#It exists, so we activate the update flag
		$update=1;
		$request->finish;
		
	}
	#Wich options enabled ?(only ipdiscover and registry implemented for the moment)
	#############
	#We read the table config looking for the ivalencap:Ethernetues of these options
	$request = $dbh->prepare("SELECT * FROM config WHERE NAME='IPDISCOVER' OR NAME='REGISTRY'");
	$request->execute();
	if($request->rows == 2){
		while($row=$request->fetchrow_hashref){
			$ipdiscover = $row->{'IVALUE'} if $row->{'NAME'}=~/IPDISCOVER/i;
			$registry = $row->{'IVALUE'} if $row->{'NAME'}=~/REGISTRY/i;
		}
	}else{
		print LOG localtime().";503;Cannot get the config keys;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";context\n";
	}
	$request->finish;
}	
#Inserting values of <HARDWARE> in hardware table
sub _hardware{
	my($YEAR, $MONTH , $DAY, $HOUR, $MIN, $SEC, $date, $request, $row, $ua, @fields);	
	
	@fields = qw/	NAME WORKGROUP OSNAME OSVERSION OSCOMMENTS PROCESSORT 
			PROCESSORS PROCESSORN MEMORY SWAP IPADDR ETIME USERID 
			TYPE DESCRIPTION WINCOMPANY WINOWNER WINPRODID/; 
	
	$ua = $r->headers_in->{'User-agent'};
	($YEAR, $MONTH , $DAY, $HOUR, $MIN, $SEC)=(localtime (time))[5,4,3,2,1,0];
	$date=sprintf "%02d:%02d:%02d:%02d:%02d:%02d", ($YEAR+1900), ($MONTH+1), $DAY, $HOUR, $MIN, $SEC;
	#We replace all data but quality and fidelity. The last come becomes the last date.
	if($update){
		for(@fields){
		  $dbh->do("UPDATE hardware set $_=".$dbh->quote($result->{CONTENT}->{HARDWARE}->{$_})." WHERE DEVICEID=".$dbh->quote($DeviceID));
		}
		$dbh->do("UPDATE hardware set USERAGENT=".$dbh->quote($ua)." WHERE DEVICEID=".$dbh->quote($DeviceID));
		$dbh->do("UPDATE hardware set LASTDATE=".$dbh->quote($date)." WHERE DEVICEID=".$dbh->quote($DeviceID));
		$dbh->do("UPDATE hardware set LASTCOME=".$dbh->quote($date)." WHERE DEVICEID=".$dbh->quote($DeviceID));
	}else{
	  	$dbh->do("INSERT INTO hardware( DEVICEID, NAME, WORKGROUP, OSNAME, OSVERSION, OSCOMMENTS, PROCESSORT, PROCESSORS, PROCESSORN, MEMORY, SWAP, IPADDR, ETIME, LASTDATE, LASTCOME, QUALITY, FIDELITY, USERID, TYPE, DESCRIPTION, WINCOMPANY, WINOWNER, WINPRODID, USERAGENT ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{HARDWARE}->{NAME}, $result->{CONTENT}->{HARDWARE}->{WORKGROUP}, $result->{CONTENT}->{HARDWARE}->{OSNAME}, $result->{CONTENT}->{HARDWARE}->{OSVERSION}, $result->{CONTENT}->{HARDWARE}->{OSCOMMENTS}, $result->{CONTENT}->{HARDWARE}->{PROCESSORT}, $result->{CONTENT}->{HARDWARE}->{PROCESSORS}, $result->{CONTENT}->{HARDWARE}->{PROCESSORN}, $result->{CONTENT}->{HARDWARE}->{MEMORY}, $result->{CONTENT}->{HARDWARE}->{SWAP}, $result->{CONTENT}->{HARDWARE}->{IPADDR}, $result->{CONTENT}->{HARDWARE}->{ETIME}, $date, $date, 0, 1, $result->{CONTENT}->{HARDWARE}->{USERID}, $result->{CONTENT}->{HARDWARE}->{TYPE}, $result->{CONTENT}->{HARDWARE}->{DESCRIPTION}, $result->{CONTENT}->{HARDWARE}->{WINCOMPANY}, $result->{CONTENT}->{HARDWARE}->{WINOWNER}, $result->{CONTENT}->{HARDWARE}->{WINPRODID}, $ua);
	}
}



#Inserting values of <ACCESSLOG> in accesslog table
sub _accesslog{
	$dbh->do("DELETE FROM accesslog WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	$dbh->do("INSERT INTO accesslog(ID, DEVICEID, USERID, LOGDATE, PROCESSES) VALUES ('', ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{ACCESSLOG}->{USERID}, $result->{CONTENT}->{ACCESSLOG}->{LOGDATE}, $result->{CONTENT}->{ACCESSLOG}->{PROCESSES});
}
#Inserting values of <BIOS> in bios table
sub _bios{
	$dbh->do("DELETE FROM bios WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	$dbh->do("INSERT INTO bios(DEVICEID, SMANUFACTURER, SMODEL, SSN, TYPE, BMANUFACTURER, BVERSION, BDATE) VALUES (?, ?, ?, ?, ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{BIOS}->{SMANUFACTURER}, $result->{CONTENT}->{BIOS}->{SMODEL}, $result->{CONTENT}->{BIOS}->{SSN}, $result->{CONTENT}->{BIOS}->{TYPE}, $result->{CONTENT}->{BIOS}->{BMANUFACTURER}, $result->{CONTENT}->{BIOS}->{BVERSION}, $result->{CONTENT}->{BIOS}->{BDATE});
}
#Inserting values of <ACCOUNTINFO> in accountinfo table
sub _accountinfo{
	#We have to look for the field's names, and its number because this table is not allways the same
	#This choice make the SQL junctions more efficient
	my ($row, $request, $accountkey, $i);
	#Fields names
        $request=$dbh->prepare("SHOW COLUMNS FROM accountinfo");
        $request->execute;
        while($row=$request->fetchrow_hashref){
                push @accountkeys, $row->{'Field'} if($row->{'Field'} ne 'DEVICEID');
        }
	#writing (if new id, but computer's update, it will be erased at the end of the execution)
	$dbh->do("INSERT INTO accountinfo(DEVICEID) VALUES(".$dbh->quote($DeviceID).")");
	#Now, we know what are the account info name fields
	#We can insert the client's data. This data will be kept in the first inventory
	#Since that the data already present of the server will be kept
	for $accountkey (@accountkeys){
		my $i=0;
		while($result->{CONTENT}->{ACCOUNTINFO}->[$i]->{KEYNAME}){
			if($result->{CONTENT}->{ACCOUNTINFO}->[$i]->{KEYNAME} eq $accountkey){
				$dbh->do("UPDATE accountinfo SET ".$accountkey."=".$dbh->quote($result->{CONTENT}->{ACCOUNTINFO}->[$i]->{KEYVALUE})." WHERE DEVICEID=".$dbh->quote($DeviceID));
			}
		$i++;
		}
	}
	$dbh->do("UPDATE accountinfo SET TAG = 'LOST' WHERE DEVICEID=".$dbh->quote($DeviceID)) if $lost;
}
#Inserting values of <REGISTRY> in registry table
sub _registry{
	$dbh->do("DELETE FROM registry WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{REGISTRY}->[$i]){
		$dbh->do("INSERT INTO registry(ID, DEVICEID, NAME, REGVALUE) VALUES('', ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{REGISTRY}->[$i]->{NAME}, $result->{CONTENT}->{REGISTRY}->[$i]->{REGVALUE});
		$i++;
	}
}
#Inserting values of <MEMORIES> in memories table
sub _memories{
	$dbh->do("DELETE FROM memories WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{MEMORIES}->[$i]){
		$dbh->do("INSERT INTO memories(ID, DEVICEID, CAPTION, DESCRIPTION, CAPACITY, PURPOSE, TYPE, SPEED, NUMSLOTS) VALUES('', ?, ?, ?, ?, ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{MEMORIES}->[$i]->{CAPTION}, $result->{CONTENT}->{MEMORIES}->[$i]->{DESCRIPTION}, $result->{CONTENT}->{MEMORIES}->[$i]->{CAPACITY}, $result->{CONTENT}->{MEMORIES}->[$i]->{PURPOSE}, $result->{CONTENT}->{MEMORIES}->[$i]->{TYPE}, $result->{CONTENT}->{MEMORIES}->[$i]->{SPEED}, $result->{CONTENT}->{MEMORIES}->[$i]->{NUMSLOTS});
		$i++;
	}
}
#Inserting values of <SLOTS> in slots table
sub _slots{
	$dbh->do("DELETE FROM slots WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{SLOTS}->[$i]){
		$dbh->do("INSERT INTO slots(ID, DEVICEID, NAME, DESCRIPTION, DESIGNATION, PURPOSE, STATUS, PSHARE) VALUES('', ?, ?, ?, ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{SLOTS}->[$i]->{NAME}, $result->{CONTENT}->{SLOTS}->[$i]->{DESCRIPTION}, $result->{CONTENT}->{SLOTS}->[$i]->{DESIGNATION}, $result->{CONTENT}->{SLOTS}->[$i]->{PURPOSE}, $result->{CONTENT}->{SLOTS}->[$i]->{STATUS}, $result->{CONTENT}->{SLOTS}->[$i]->{SHARED});
		$i++;
	}
}
#Inserting values of <CONTROLLERS> in controllers table
sub _controllers{
	$dbh->do("DELETE FROM controllers WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{CONTROLLERS}->[$i]){
		$dbh->do("INSERT INTO controllers(ID, DEVICEID, MANUFACTURER, NAME, CAPTION, DESCRIPTION, VERSION, TYPE) VALUES('', ?, ?, ?, ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{CONTROLLERS}->[$i]->{MANUFACTURER}, $result->{CONTENT}->{CONTROLLERS}->[$i]->{NAME}, $result->{CONTENT}->{CONTROLLERS}->[$i]->{CAPTION}, $result->{CONTENT}->{CONTROLLERS}->[$i]->{DESCRIPTION},  $result->{CONTENT}->{CONTROLLERS}->[$i]->{VERSION}, $result->{CONTENT}->{CONTROLLERS}->[$i]->{TYPE});
		$i++;
	}
}
#Inserting values of <MONITORS> in monitors table
sub _monitors{
	$dbh->do("DELETE FROM monitors WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{MONITORS}->[$i]){
		$dbh->do("INSERT INTO monitors(ID, DEVICEID, MANUFACTURER, CAPTION, DESCRIPTION, TYPE, SERIAL) VALUES('', ?, ?, ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{MONITORS}->[$i]->{MANUFACTURER}, $result->{CONTENT}->{MONITORS}->[$i]->{CAPTION}, $result->{CONTENT}->{MONITORS}->[$i]->{DESCRIPTION}, $result->{CONTENT}->{MONITORS}->[$i]->{TYPE}, $result->{CONTENT}->{MONITORS}->[$i]->{SERIAL});
		$i++;
	}
}
#Inserting values of <PORTS> in ports table
sub _ports{
	$dbh->do("DELETE FROM ports WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{PORTS}->[$i]){
		$dbh->do("INSERT INTO ports(ID, DEVICEID, TYPE, NAME, CAPTION, DESCRIPTION) VALUES('', ?, ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{PORTS}->[$i]->{TYPE}, $result->{CONTENT}->{PORTS}->[$i]->{NAME}, $result->{CONTENT}->{PORTS}->[$i]->{CAPTION}, $result->{CONTENT}->{PORTS}->[$i]->{DESCRIPTION});
		$i++;
	}
}
#Inserting values of <STORAGES> in storages table
sub _storages{
	$dbh->do("DELETE FROM storages WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{STORAGES}->[$i]){
		$dbh->do("INSERT INTO storages(ID, DEVICEID, MANUFACTURER, NAME, MODEL, DESCRIPTION, TYPE, DISKSIZE) VALUES('', ?, ?, ?, ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{STORAGES}->[$i]->{MANUFACTURER}, $result->{CONTENT}->{STORAGES}->[$i]->{NAME}, $result->{CONTENT}->{STORAGES}->[$i]->{MODEL}, $result->{CONTENT}->{STORAGES}->[$i]->{DESCRIPTION}, $result->{CONTENT}->{STORAGES}->[$i]->{TYPE}, $result->{CONTENT}->{STORAGES}->[$i]->{DISKSIZE},);
		$i++;
	}
}
#Inserting values of <DRIVES> in drives table
sub _drives{
	$dbh->do("DELETE FROM drives WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{DRIVES}->[$i]){
		$dbh->do("INSERT INTO drives(ID, DEVICEID, LETTER, TYPE, FILESYSTEM, TOTAL, FREE, NUMFILES, VOLUMN) VALUES('', ?, ?, ?, ?, ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{DRIVES}->[$i]->{LETTER}, $result->{CONTENT}->{DRIVES}->[$i]->{TYPE}, $result->{CONTENT}->{DRIVES}->[$i]->{FILESYSTEM}, $result->{CONTENT}->{DRIVES}->[$i]->{TOTAL}, $result->{CONTENT}->{DRIVES}->[$i]->{FREE}, $result->{CONTENT}->{DRIVES}->[$i]->{NUMFILES}, $result->{CONTENT}->{DRIVES}->[$i]->{VOLUMN});
		$i++;
	}
}
#Inserting values of <INPUTS> in inputs table
sub _inputs{
	$dbh->do("DELETE FROM inputs WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{INPUTS}->[$i]){
		$dbh->do("INSERT INTO inputs(ID, DEVICEID, TYPE, MANUFACTURER, CAPTION, DESCRIPTION, INTERFACE, POINTTYPE) VALUES('', ?, ?, ?, ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{INPUTS}->[$i]->{TYPE}, $result->{CONTENT}->{INPUTS}->[$i]->{MANUFACTURER}, $result->{CONTENT}->{INPUTS}->[$i]->{CAPTION}, $result->{CONTENT}->{INPUTS}->[$i]->{DESCRIPTION}, $result->{CONTENT}->{INPUTS}->[$i]->{INTERFACE}, $result->{CONTENT}->{INPUTS}->[$i]->{POINTTYPE});
		$i++;
	}
}
#Inserting values of <MODEMS> in modems table
sub _modems{
	$dbh->do("DELETE FROM modems WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{MODEMS}->[$i]){
		$dbh->do("INSERT INTO modems(ID, DEVICEID, NAME, MODEL, DESCRIPTION, TYPE) VALUES('', ?, ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{MODEMS}->[$i]->{NAME}, $result->{CONTENT}->{MODEMS}->[$i]->{MODEL}, $result->{CONTENT}->{MODEMS}->[$i]->{DESCRIPTION}, $result->{CONTENT}->{MODEMS}->[$i]->{TYPE});
		$i++;
	}
}
#Inserting values of <NETWORKS> in networks table
sub _networks{
	$dbh->do("DELETE FROM networks WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{NETWORKS}->[$i]){
		$dbh->do("INSERT INTO networks(ID, DEVICEID, DESCRIPTION, TYPE, TYPEMIB, SPEED, MACADDR, STATUS, IPADDRESS, IPMASK, IPGATEWAY, IPSUBNET, IPDHCP) VALUES('', ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{NETWORKS}->[$i]->{DESCRIPTION}, $result->{CONTENT}->{NETWORKS}->[$i]->{TYPE}, $result->{CONTENT}->{NETWORKS}->[$i]->{TYPEMIB}, $result->{CONTENT}->{NETWORKS}->[$i]->{SPEED}, $result->{CONTENT}->{NETWORKS}->[$i]->{MACADDR}, $result->{CONTENT}->{NETWORKS}->[$i]->{STATUS}, $result->{CONTENT}->{NETWORKS}->[$i]->{IPADDRESS}, $result->{CONTENT}->{NETWORKS}->[$i]->{IPMASK}, $result->{CONTENT}->{NETWORKS}->[$i]->{IPGATEWAY}, $result->{CONTENT}->{NETWORKS}->[$i]->{IPSUBNET}, $result->{CONTENT}->{NETWORKS}->[$i]->{IPDHCP});
		$i++;
	}
}
#Inserting values of <PRINTERS> in printers table
sub _printers{
	$dbh->do("DELETE FROM printers WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{PRINTERS}->[$i]){
		$dbh->do("INSERT INTO printers(ID, DEVICEID, NAME, DRIVER, PORT) VALUES('', ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{PRINTERS}->[$i]->{NAME}, $result->{CONTENT}->{PRINTERS}->[$i]->{DRIVER}, $result->{CONTENT}->{PRINTERS}->[$i]->{PORT});
		$i++;
	}
}
#Inserting values of <SOUNDS> in sounds table
sub _sounds{
	$dbh->do("DELETE FROM sounds WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{SOUNDS}->[$i]){
		$dbh->do("INSERT INTO sounds(ID, DEVICEID, MANUFACTURER, NAME, DESCRIPTION) VALUES('', ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{SOUNDS}->[$i]->{MANUFACTURER}, $result->{CONTENT}->{SOUNDS}->[$i]->{NAME}, $result->{CONTENT}->{SOUNDS}->[$i]->{DESCRIPTION});
		$i++;
	}
}
#Inserting values of <VIDEOS> in videos table
sub _videos{
	$dbh->do("DELETE FROM videos WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{SOUNDS}->[$i]){
		$dbh->do("INSERT INTO videos(ID, DEVICEID, NAME, CHIPSET, MEMORY, RESOLUTION) VALUES('', ?, ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{VIDEOS}->[$i]->{NAME}, $result->{CONTENT}->{VIDEOS}->[$i]->{CHIPSET}, $result->{CONTENT}->{VIDEOS}->[$i]->{MEMORY}, $result->{CONTENT}->{VIDEOS}->[$i]->{RESOLUTION});
		$i++;
	}
}
#Inserting values of <SOFTWARES> in softwares table
sub _softwares{
	$dbh->do("DELETE FROM softwares WHERE DEVICEID=".$dbh->quote($DeviceID)) if($update);
	my $i=0;
	while($result->{CONTENT}->{SOFTWARES}->[$i]){
		$dbh->do("INSERT INTO softwares(ID, DEVICEID, PUBLISHER, NAME, VERSION, FOLDER, COMMENTS, FILENAME, FILESIZE, SOURCE) VALUES('', ?, ?, ?, ?, ?, ?, ?, ?, ?)", {}, $DeviceID, $result->{CONTENT}->{SOFTWARES}->[$i]->{PUBLISHER}, $result->{CONTENT}->{SOFTWARES}->[$i]->{NAME}, $result->{CONTENT}->{SOFTWARES}->[$i]->{VERSION}, $result->{CONTENT}->{SOFTWARES}->[$i]->{FOLDER}, $result->{CONTENT}->{SOFTWARES}->[$i]->{COMMENTS}, $result->{CONTENT}->{SOFTWARES}->[$i]->{FILENAME}, $result->{CONTENT}->{SOFTWARES}->[$i]->{FILESIZE}, $result->{CONTENT}->{SOFTWARES}->[$i]->{SOURCE});
		$i++;
	}
}
#To manage the update request
sub _update{
	my %resp;
	my @agent;
	my @dmi;
	my @ipdiscover;
	my $Acurrent;
	my $Dcurrent;
	my $Icurrent;
	my $Iversion;
	my $agent;
	my $dmi;
	my $ip;
	my $platform;
	my $Dversion;
	my $Aversion;
	my $request;
	my $row;
	
	#Parse the request
	#OS type
	$platform = $query->{PLATFORM};
	#Version of the agent
	$Aversion = $query->{AGENT};
	#Eventually, the DMI version
	if(defined($query->{DMI})){$Dversion = $query->{DMI}};
	if(defined($query->{IPDISCOVER})){$Iversion = $query->{IPDISCOVER}};
	if(!defined($Aversion) || !($platform=~/^WINDOWS$|^MAC$|^LINUX$/)){
			print LOG localtime().";508;Update informations unreadable;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";update\n";
			return Apache2::Const::HTTP_BAD_REQUEST;
	}
	
	#What are the available versions in the database
	$request=$dbh->prepare("SELECT * FROM files WHERE OS=".$dbh->quote($platform));
	$request->execute;
	#If no file available, tell to the clien t not to update
	unless($request->rows){
		&_send_response({ 'RESPONSE' => ['NO_UPDATE'] });
		$request->finish;
		return Apache2::Const::OK;
	}else{
		#Files are presents, does the client have to download and install it ?
		#Get versions number
		while($row=$request->fetchrow_hashref()){
			#Version of the agent in the database
			if($row->{'NAME'}=~/agent/i){
				push @agent, $row->{'VERSION'};
			}
			#Maybe a dmi reader version(on a linux computer)
			if(defined($Dversion)){
				#Version of the dmi in the database
				if($row->{'NAME'}=~/dmi/i){
					push @dmi, $row->{'VERSION'};
				}
			}
			if(defined($Iversion)){
				#Version of the dmi in the database
				if($row->{'NAME'}=~/ipdiscover/i){
					push @ipdiscover, $row->{'VERSION'};
				}
			}
		}
		#Determine the upper agent version available	
		if(@agent){
			#Looking for the latest version
			$Acurrent=0;
			for(@agent){
				if($_>$Acurrent){$Acurrent=$_;}
			}
			#Compare to the client version. If different, we tell you to update
			$agent=($Aversion==$Acurrent)?0:1;
		}else{
			$agent=0;
		}
		
		#DMI
		$Dcurrent=0;
		if(defined($Dversion) and @dmi){
			for(@dmi){
				if($_>$Dcurrent){$Dcurrent=$_;}
			}
			#Compare to the client version. If different, we tell you to update
			$dmi=($Dversion==$Dcurrent)?0:1;
		}
		#IPDISCOVER
		if(defined($Iversion) and @ipdiscover){
			for(@ipdiscover){
				if($_>$Icurrent){$Icurrent=$_;}
			}
			#Compare to the client version. If different, we tell you to update
			$ip=($Iversion==$Icurrent)?0:1;
		}
	}
	$request->finish;
	#Generate the response
	unless(($agent) or ($dmi) or $ip){
		&_send_response({'RESPONSE',['NO_UPDATE']});
		return Apache2::Const::OK;
	}
	$resp{'RESPONSE'} = ['UPDATE'];
	if( $agent ){ $resp{'AGENT'} = [ $Acurrent ] }
	if( $dmi ){ $resp{'DMI'} = [ $Dcurrent ] }
	if( $ip ){ $resp{'IPDISCOVER'} = [ $Icurrent ] }
	#Send it
	&_send_response(\%resp);
	return Apache2::Const::OK;
}
	

#Subroutine called at the end of database inventory insertions
sub _postinsert{
	my $device;
	my $Macaddress;
	my $i;
	my %exist;
	my $table;
	my $request;
	my $row;
	my $resp;
	my $accountkey;
	my %elements;
	
	#If the duplicate is specified
	if($result->{CONTENT}->{OLD_DEVICEID}){
		my $flag = $dbh->do("SELECT * FROM hardware WHERE DEVICEID=".$dbh->quote($result->{CONTENT}->{OLD_DEVICEID}));
		if($flag){
		  &_duplicate($result->{CONTENT}->{OLD_DEVICEID});
		  $red=1;
		}
	#If not, we try to manage a true duplicate	
	}else{
		#Have we already got the hostname
		$request = $dbh->prepare("SELECT DEVICEID, NAME FROM hardware WHERE NAME= '$result->{CONTENT}->{HARDWARE}->{NAME}'");
		$request->execute();
		while($row = $request->fetchrow_hashref()){
			if(!($row->{'NAME'} eq '') and !($row->{'DEVICEID'} eq $DeviceID)){
				$exist{$row->{'DEVICEID'}}{'HOSTNAME'}=1;
			}
		}
		#...and one MAC of this machine
		$i=0;
		while($result->{CONTENT}->{NETWORKS}->[$i]->{MACADDR}){
			$Macaddress=$result->{CONTENT}->{NETWORKS}->[$i]->{MACADDR};
			$request = $dbh->prepare("SELECT DEVICEID,DESCRIPTION,MACADDR FROM networks WHERE MACADDR=".$dbh->quote($Macaddress));
			$request->execute();
			while($row = $request->fetchrow_hashref()){
				if(!($row->{'MACADDR'} eq '') and !($row->{'DEVICEID'} eq $DeviceID)){
					$exist{$row->{'DEVICEID'}}{'MACADDRESS'}++ unless (($row->{'MACADDR'}=~/00:00:00:00:00:00/i) or ($row->{'MACADDR'}=~/FF:FF:FF:FF:FF:FF/i));
				}
			}
			$i++;
		}
		#We delete the entries created while parsing the xml
		delete $result->{CONTENT}->{NETWORKS}->[$i];
		#...or its serial
		$request = $dbh->prepare("SELECT DEVICEID, SSN FROM bios WHERE SSN=".$dbh->quote($result->{CONTENT}->{BIOS}->{SSN}));
		$request->execute();
		while($row = $request->fetchrow_hashref()){
			if(!($row->{'SSN'} eq '') and !($row->{'DEVICEID'} eq $DeviceID)){
				$exist{$row->{'DEVICEID'}}{'SSN'}=1;
			}
		}
		$request->finish;
		
		#If we have in the same time the same serial, the same hostname and at least one hardware address in the database, we consider that it's a double(log : 300). If we have one of these three elements, we say that it is maybe a double, and we log a 301 event.
	
		for $device (keys(%exist)){
			if($exist{$device}->{'HOSTNAME'} and $exist{$device}->{'MACADDRESS'} and $exist{$device}->{'SSN'}){
				$red=1;
				&_duplicate($device);
			}	
		}
	}
	#We verify accountinfo diff if the machine was already in the database
	if($update or $red){
		#We put back the account infos to the agent if necessary
		$request=$dbh->prepare("SELECT * FROM accountinfo WHERE DEVICEID=".$dbh->quote($DeviceID));
		$request->execute;
		if($row=$request->fetchrow_hashref()){
			my $up=0;
			#Compare the account infos with the user's ones
			for $accountkey (@accountkeys){
				$i=0;
				while($result->{CONTENT}->{ACCOUNTINFO}->[$i]){
					if($result->{CONTENT}->{ACCOUNTINFO}->[$i]->{KEYNAME} eq $accountkey){
						$up=1,last if($result->{CONTENT}->{ACCOUNTINFO}->[$i]->{KEYVALUE} ne $row->{$accountkey});
					}
				$i++;
				}
			}
			#If there is something new in the table
			$up=1 if(@accountkeys != $i);
			
			if($up){
				#we write the xml data
				$elements{'RESPONSE'} = [ 'ACCOUNT_UPDATE' ];
				my $i=0;
				for(@accountkeys){
					$elements{'ACCOUNTINFO'}[$i]{'KEYNAME'} = [ $_ ];
					$elements{'ACCOUNTINFO'}[$i]{'KEYVALUE'} = [ $row->{$_} ];
					$i++;
				}
				$request->finish;
				
				#send the update to the client
				&_send_response(\%elements);
				return;
			}else{
				$request->finish;
				&_send_response({'RESPONSE' => [ 'NO_ACCOUNT_UPDATE' ]});
				return;
			}
		
		}else{
		#There is a problem. The device MUST be present in the table
			print LOG localtime().";509;Cannot get account infos;".$DeviceID.";".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";postinsert\n";
			$request->finish;
			$elements{'RESPONSE'} = [ 'ACCOUNT_UPDATE' ];
                        my $i=0;
                        while($result->{CONTENT}->{ACCOUNTINFO}->[$i]->{KEYNAME}){
			  if($result->{CONTENT}->{ACCOUNTINFO}->[$i]->{KEYNAME} eq 'TAG'){
			    $elements{'ACCOUNTINFO'}[$i]{'KEYVALUE'} = [ 'LOST' ];
			  }else{
                            $elements{'ACCOUNTINFO'}[$i]{'KEYVALUE'} = [ $result->{CONTENT}->{ACCOUNTINFO}->[$i]->{KEYVALUE} ];
			  }
			  $elements{'ACCOUNTINFO'}[$i]{'KEYNAME'} = [ $result->{CONTENT}->{ACCOUNTINFO}->[$i]->{KEYNAME} ];
			  $i++;
			}
			#lost flag
			$lost=1;
			#call accountinfo to insert agent values in addition to the LOST value in TAG
			&_accountinfo();
			#send the update to the client with TAG=LOST
                        &_send_response(\%elements);
			return;
		}
	
	}else{
		&_send_response({'RESPONSE' => [ 'NO_ACCOUNT_UPDATE' ]});
		return;
	}	
}
#Subroutine wich answer to client prolog
sub _prolog_resp{
	my %resp;
	my $deviceID;
	my @param;
	my $OS;
	my $CVersion;
	my $DMIVersion;
	my $row;
	my $request;
	my $frequency;
	my $CurrentDMI;
	my $CurrentCL;
	my ($YEAR, $MONTH , $DAY, $HOUR, $MIN, $SEC, $NOW, $LAST);
	my $lastcome;
	my $lastdate;
	my $quality;
	my $flag;
	#Get the DEVICEID of the sender
	$deviceID=$query->{DEVICEID};
	unless(defined($deviceID)){
		print LOG localtime().";501;Cannot read device ID;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";prolog\n";
		return Apache2::Const::HTTP_BAD_REQUEST;
	}
	#Verify it
	unless($deviceID=~/.+-\d{4}-\d{2}-\d{2}-\d{2}-\d{2}-\d{2}/){
		print LOG localtime().";502;Device ID $deviceID invalid;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";prolog\n";
		return Apache2::Const::HTTP_BAD_REQUEST;
	}
	#Default inventory update frequency (in days)
	$request=$dbh->prepare("SELECT IVALUE FROM config WHERE NAME='Frequency'");
	$request->execute;
	if($row=$request->fetchrow_hashref()){
		$frequency = $row->{'IVALUE'}; 
	}
	$request->finish;
	#If we do not have the default frequency
	unless(defined($frequency)){
		
		print LOG localtime().";503;Cannot get the default frequency value;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";prolog\n";
		$resp{'RESPONSE'} = ['STOP'];
		&_send_response(\%resp);
		return Apache2::Const::OK;
	}
	
	###########
	#If it is an existing machine, we write its quality number (number of days+1 between the two last inventory)A zero value means a recently inventoried machine. The "fidelity" field gives the power of the existing quality
	##########################################################
	$request = $dbh->prepare('SELECT DEVICEID,LASTCOME,QUALITY,FIDELITY FROM hardware WHERE DEVICEID='.$dbh->quote($deviceID));
	$request->execute;
	#The machine exists
	if($request->rows){
		#To know that the deviceid exists in the database
		$flag=1;
		$row = $request->fetchrow_hashref;
		#Date of the day
		($YEAR, $MONTH , $DAY, $HOUR, $MIN, $SEC)=(localtime (time))[5,4,3,2,1,0];
		$MONTH++;
		#In seconds
		$NOW=((525200*($YEAR))+(43200*($MONTH))+(1440*$DAY)+($HOUR*60)+$MIN)*60+$SEC;
		my $daydate = sprintf "%04d:%02d:%02d:%02d:%02d:%02d", ($YEAR+1900), ($MONTH), $DAY, $HOUR, $MIN, $SEC;
		#Date of the last visit
		$lastcome = $row->{'LASTCOME'};
		if($lastcome=~/(\d{4})-(\d{2})-(\d{2})\s+(\d{2}):(\d{2}):(\d{2})/){
			$YEAR=$1-1900;($MONTH)=$2;$DAY=$3;$HOUR=$4;$MIN=$5;$SEC=$6;
		}
		$LAST=((525200*($YEAR))+(43200*$MONTH)+(1440*$DAY)+($HOUR*60)+$MIN)*60+$SEC;
		$request->finish;
		#Calculate quality 
		if($row->{'FIDELITY'} > 1){
			$quality = ((($NOW-$LAST)/86400) + ($row->{'QUALITY'}*$row->{'FIDELITY'}))/(($row->{'FIDELITY'})+1);
			$dbh->do('UPDATE hardware SET FIDELITY=FIDELITY+1 WHERE DEVICEID='.$dbh->quote($deviceID));
		}else{
		#We increment the number of visits
			$quality = (($NOW-$LAST)/86400);
			$dbh->do('UPDATE hardware SET FIDELITY=FIDELITY+1 WHERE DEVICEID='.$dbh->quote($deviceID));
		}
		$dbh->do('UPDATE hardware SET QUALITY='.$quality.' WHERE DEVICEID='.$dbh->quote($deviceID));
		$dbh->do('UPDATE hardware SET LASTCOME='.$dbh->quote($daydate).' WHERE DEVICEID='.$dbh->quote($deviceID));	
	}
	
	##########
	#If special value 0, we allways accept
	if($frequency==0){
		$resp{'RESPONSE'} = ['SEND'];
		&_options(\%resp, $deviceID);
		&_send_response(\%resp);
		print LOG localtime().";100;$deviceID accepted - General frequency is set to ALLWAYS;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";prolog\n" if $loglevel;
		return Apache2::Const::OK;
	#If -1, we never accept
	}elsif($frequency==(-1)){
		$resp{'RESPONSE'} = ['STOP'];
		print LOG localtime().";102;$deviceID refused - General frequency is set to NEVER;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";prolog\n" if $loglevel;
		&_send_response(\%resp);
		return Apache2::Const::OK;
	}
	
	#Verifiy that deviceid exist
	unless($flag){
	#If not, we accept the inventory
		$resp{'RESPONSE'} = ['SEND'];
		&_options(\%resp, $deviceID);
		&_send_response(\%resp);
		print LOG localtime().";103;$deviceID accepted - New DeviceID;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";prolog\n" if $loglevel;
		$request->finish;
		return Apache2::Const::OK;
	}
	$request->finish;
		
	#Maybe there are computer's defined parameter
	$request=$dbh->prepare("SELECT IVALUE FROM devices WHERE DEVICEID=".$dbh->quote($deviceID)."AND NAME='FREQUENCY'");
	$request->execute;
	if($row=$request->fetchrow_hashref()){
		$frequency=$row->{'IVALUE'};
	}
	$request->finish;
	
	#If ipdiscover enable
	$request=$dbh->prepare("SELECT IVALUE FROM devices WHERE DEVICEID=".$dbh->quote($deviceID)."AND NAME='IPDISCOVER'");
	$request->execute;
	$frequency = 0 if $request->rows;
	$request->finish;
		
	#If special values...
	if($frequency==0){
		$resp{'RESPONSE'} = ['SEND'];
		&_options(\%resp, $deviceID);
		&_send_response(\%resp);
		print LOG localtime().";100;$deviceID accepted - Special frequency is set to ALLWAYS;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";prolog\n" if $loglevel;
		return Apache2::Const::OK;
	}elsif($frequency==(-1)){
		$resp{'RESPONSE'} = ['STOP'];
		print LOG localtime().";102;$deviceID refused - Special frequency is set to NEVER;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";prolog\n" if $loglevel;
		&_send_response(\%resp);
		return Apache2::Const::OK;
	}
	
	#Getting the last date
	$request=$dbh->prepare("SELECT LASTDATE FROM hardware WHERE DEVICEID=".$dbh->quote($deviceID));
	$request->execute;
	unless ($request->rows){
		print LOG localtime().";504;Cannot get last inventory date for $deviceID;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";prolog\n";
		$resp{'RESPONSE'} = ['SEND'];
		&_options(\%resp, $deviceID);
		&_send_response(\%resp);
		return Apache2::Const::OK;
	}else{
		if($row=$request->fetchrow_hashref()){
			$lastdate=$row->{'LASTDATE'};
		}
		if($lastdate=~/(\d{4})-(\d{2})-(\d{2})\s+(\d{2}):(\d{2}):(\d{2})/){
			$YEAR=$1-1900;$MONTH=$2;$DAY=$3;$HOUR=$4;$MIN=$5;$SEC=$6;
		}
		$LAST=((525200*($YEAR))+(43200*($MONTH)+(1440*$DAY)+($HOUR*60)+$MIN))*60+$SEC;
	}
	$request->finish;
	#Have we override the period ?
	my $time = (($NOW-$LAST)+$frequency*86400);	
	if((($LAST-$NOW)+$frequency*86400)<0){
		$resp{'RESPONSE'} = ['SEND'];
		&_options(\%resp, $deviceID);
		&_send_response(\%resp);
		print LOG localtime().";100;$deviceID accepted - update inventory;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";prolog\n" if $loglevel;
		return Apache2::Const::OK;
	}else{
		$resp{'RESPONSE'} = ['STOP'];
		print LOG localtime().";102;$deviceID refused - Device up to date;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";prolog\n" if $loglevel;
		&_send_response(\%resp);
		return Apache2::Const::OK;
	}
}

sub _options{
###############################
#inventories options management
###############################
	my ($request, $row, $i, $p, $deviceID, $ref);
	
	$ref = shift;
	$deviceID = shift;
	
	################################
	#IPDISCOVER
	###########
	#What is the current state of this option ?
	$request=$dbh->prepare("SELECT * FROM config WHERE NAME='IPDISCOVER'");
	$request->execute;
	if($request->rows){
		$row = $request->fetchrow_hashref();
		if($row->{'IVALUE'} != 0){	
			#ipdiscover for this device ?
			$request=$dbh->prepare('SELECT * FROM devices WHERE DEVICEID='.$dbh->quote($deviceID).' AND NAME="IPDISCOVER"');
			$request->execute();
			if($request->rows){
				$row = $request->fetchrow_hashref();
				$$ref{'OPTION'}[$i]{'NAME'} = [ 'IPDISCOVER' ];
				$$ref{'OPTION'}[$i]{'PARAM'}[$p] = $row->{'TVALUE'};
				$i ++;
				$r->err_headers_out->{'Connection'} =  'close';
			}		
		}
	}
	#################################
	#REGISTRY
	#########
	$request=$dbh->prepare("SELECT * FROM config WHERE NAME='REGISTRY'");
	$request->execute;
	if($request->rows){
		$row = $request->fetchrow_hashref();
		if($row->{'IVALUE'} == 1){
			$request=$dbh->prepare("SELECT * FROM regconfig");
			$request->execute;
			$$ref{'OPTION'}[$i]{'NAME'} = [ 'REGISTRY' ] if $request->rows;
			for(0..$request->rows-1){
				$row=$request->fetchrow_hashref;
				$$ref{'OPTION'}[$i]{'PARAM'}[$p] = {'REGTREE' => $row->{'REGTREE'}, 
								    'REGKEY' => $row->{'REGKEY'},
								    'NAME' => $row->{'NAME'},
								    'content' => $row->{'REGVALUE'}
								     };
				$p++;
			}
		}
	}
	$i++;#For the next option...
	return;
}



sub _ipdiscover{
	my($request, $row, $need_relay, $remove, $net, $handlers, $mask, $netid, $qual, $fid, $val);
######################
#ipdiscover management
######################
	#
	############
	#THE SUBNET
	############
	my $ifn = 0;
	my $netnum=0;
	while($result->{CONTENT}->{NETWORKS}->[$ifn]){
		$netnum = $result->{CONTENT}->{NETWORKS}->[$ifn]->{IPSUBNET} unless $result->{CONTENT}->{NETWORKS}->[$ifn]->{DESCRIPTION}=~/ppp/i or !$result->{CONTENT}->{NETWORKS}->[$ifn]->{STATUS}=~/up/i;
		if ($netnum =~ /^(\d{1,3}(?:\.\d{1,3}){3})$/){
			#The subnet is it full of relay ?(IVALUE parameter in config table for the number for desired hosts, wich will be compared to the tvalues of devices in the devices table)
			  $request=$dbh->prepare('SELECT deviceid FROM devices WHERE TVALUE='.$dbh->quote($netnum).' AND NAME="IPDISCOVER"' );
			  $request->execute();
			  $row = $request->fetchrow_hashref;
			  #If not, $need_relay = 1
			  $handlers = $request->rows;
			  $need_relay=1 if $handlers < $ipdiscover;
			  #If the gateway is full of relays, we try to determine an other gateway for the device
			  $ifn++,next unless($need_relay);
			  $request->finish;
			  #			
			  last;
		}
	$ifn ++;	
	}
	
	#No valid subnet
	return unless $netnum =~ /^(\d{1,3}(?:\.\d{1,3}){3})$/;
	
	############
	#THE MACHINE
	############
	#
	#Is it a good computer ?
	$request = $dbh->prepare('SELECT QUALITY,FIDELITY FROM hardware WHERE DEVICEID='.$dbh->quote($DeviceID));
	$request->execute();
	#
	if($row = $request->fetchrow_hashref){
	  $qual = $row->{'QUALITY'};
	  $fid = $row->{'FIDELITY'};
	}
	$request->finish;
	
	#Is the device already have the ipdiscover function ?
	$request=$dbh->prepare('SELECT TVALUE FROM devices WHERE DEVICEID='.$dbh->quote($DeviceID).' AND NAME="IPDISCOVER"');
	$request->execute();
	#if it has ipdiscover functionnality, does it do good work ?
	if($request->rows){
		$row = $request->fetchrow_hashref;
		$net = $row->{'TVALUE'};
		$request->finish;
		#Quality
		$remove = 1 if($handlers > $ipdiscover);
		if(exists($result->{CONTENT}->{IPDISCOVER})){
			#We insert the results (MAC/IP)
			my $i = 0;
			$request = $dbh->prepare('SELECT IPMASK,IPSUBNET FROM devices,networks where tvalue=ipsubnet and devices.deviceid=networks.deviceid and networks.deviceid='.$dbh->quote($result->{CONTENT}->{DEVICEID}));
			$request->execute;
			$row = $request->fetchrow_hashref;
			$mask = $row->{'IPMASK'};
			$netid = $row->{'IPSUBNET'};
			while($result->{CONTENT}->{IPDISCOVER}->{H}->[$i]){
				unless($result->{CONTENT}->{IPDISCOVER}->{H}->[$i]->{I}=~/^(\d{1,3}(?:\.\d{1,3}){3})$/ 
				  and $result->{CONTENT}->{IPDISCOVER}->{H}->[$i]->{M}=~/.{2}(?::.{2}){5}/){
				    print LOG localtime().";303;bad result(".$result->{CONTENT}->{IPDISCOVER}->{H}->[$i]->{M}." => ".$result->{CONTENT}->{IPDISCOVER}->{H}->[$i]->{I}."), skipping.;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";ipdiscover\n" if $loglevel;
				    $i++;
				    next;
				}
				$request = $dbh->prepare('UPDATE netmap SET IP='.$dbh->quote($result->{CONTENT}->{IPDISCOVER}->{H}->[$i]->{I}).',MASK='.$dbh->quote($mask).',NETID='.$dbh->quote($netid).',DATE=NULL WHERE MAC='.$dbh->quote($result->{CONTENT}->{IPDISCOVER}->{H}->[$i]->{M}));
				$request->execute;
				$dbh->do('INSERT INTO netmap(IP, MAC, MASK, DATE, NETID) values(?,?,?,NULL,?)',{},$result->{CONTENT}->{IPDISCOVER}->{H}->[$i]->{I}, $result->{CONTENT}->{IPDISCOVER}->{H}->[$i]->{M}, $mask, $netid) unless($request->rows);
				$i++;
			}
		}else{$remove = 1}
		#If the gateway do not still exist, we deactivate the feature
		$request = $dbh->prepare('SELECT * FROM networks WHERE IPSUBNET='.$dbh->quote($net));
		$request->execute();
		$remove = 1 unless $request->rows;
		$request->finish;
		
	#relay -- if no ip or bad quality
	}elsif($need_relay){
		#if it has not, is it able to ?(Operating System, Network Sub Mask)
		#Quality
		if($qual!=0 and $fid > 2){
			if($result->{CONTENT}->{HARDWARE}->{OSNAME}=~/xp|2000|linux/i){
				#The ipdiscover function will be available until class B network (not more)
				if($result->{CONTENT}->{NETWORKS}->[$ifn]->{IPMASK}=~/255\.255\.\d{1,3}\.\d{1,3}/){
					#if it is compliant with this function, activate it in devices table
					$dbh->do('INSERT INTO devices(DEVICEID, NAME, IVALUE, TVALUE, COMMENTS) VALUES(?,?,?,?,?)',{},$DeviceID,'IPDISCOVER',1,$result->{CONTENT}->{NETWORKS}->[$ifn]->{IPSUBNET},'');
					print LOG localtime().";303;IPDISCOVER enabled for $DeviceID;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";ipdiscover\n" if $loglevel;
				}
			}
		}
		$request->finish;
	}else{
	#We look at the quality of the current computer. If it is better than the worth ipdiscover of that gateway, we replace it
		#Is the computer able to become ipdiscover
		my @worth;
		my $time = time();
		#Default inventory update frequency (in days)
		$request = $dbh->prepare("SELECT IVALUE FROM config WHERE NAME='IPDISCOVER_MAX_ALIVE'");
		$request->execute;
		if($row = $request->fetchrow_hashref()){
			$val = ($row->{'IVALUE'} * 86400); 
		}
		$request->finish;
		if($qual!=0 and $fid > 2){
        		if($result->{CONTENT}->{HARDWARE}->{OSNAME}=~/xp|2000|linux/i){
				#Looking for the worth value
				$request = $dbh->prepare('select h.DEVICEID AS DEVICEID, h.QUALITY AS QUALITY, UNIX_TIMESTAMP(h.LASTDATE) AS LAST from hardware h,devices d where d.DEVICEID=h.DEVICEID and d.TVALUE='.$dbh->quote($netnum).' AND h.DEVICEID<>'.($dbh->quote($DeviceID)));
				$request->execute;
				my $over;
				while($row = $request->fetchrow_hashref){
					#If we find an ipdiscover that is older than IP_MAX_ALIVE, we replace it with the current
					if( (($time - $row->{'LAST'}) > $val) && $val){
						@worth = ($row->{'DEVICEID'}, $row->{'QUALITY'} );
						$over = 1;
						last;						
					}
					#For the first round
					unless(@worth){
					  @worth = ($row->{'DEVICEID'}, $row->{'QUALITY'} );
					  next;
					}
					#Put the worth in @worth
					@worth = ( $row->{'DEVICEID'}, $row->{'QUALITY'} ) if $worth[1] < $row->{'QUALITY'};
				}
				if(($qual < $worth[1] and ($worth[1]-$qual>1)) or $over){
				  #Compare to the current and replace it if needed
				  $dbh->do('UPDATE devices SET DEVICEID='.($dbh->quote($DeviceID)).' where DEVICEID='.($dbh->quote($worth[0])).' AND (NAME="IPDISCOVER")'); 
				  print LOG localtime().";303;IPDISCOVER enabled for $DeviceID (in replacement of ".$worth[0]." -> ".($over?'Time over':'Better quality').");".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";ipdiscover\n" if $loglevel;
				}
	    		}
	  	}
	}
	#If needed, we remove
	if($remove and !$red){
		$dbh->do('DELETE FROM devices WHERE DEVICEID='.$dbh->quote($DeviceID).' AND NAME="IPDISCOVER"');
		print LOG localtime().";303;IPDISCOVER disabled for $DeviceID;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";ipdiscover\n" if $loglevel;		
	}
}



sub _send_response{
	my( $xml, $message, $d, $status );
	#Generate the response
	#Generation of xml message
	$message=XML::Simple::XMLout( $_[0], RootName => 'REPLY', XMLDecl => '<?xml version="1.0" encoding="ISO-8859-1"?>',
	                 NoSort => 1, SuppressEmpty => undef);
	#send
	$message=Compress::Zlib::compress( $message ) or print LOG localtime().";506;Compression problem;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";send_response\n", return Apache2::Const::HTTP_BAD_REQUEST;
	$r->err_headers_out->{'Content-length'} = length($message);
	$r->err_headers_out->{'Cache-control'} = 'no-cache';
	$r->content_type('application/x-compressed');
	$r->print($message);
	return Apache2::Const::OK;
}

sub _duplicate{
	my @tables=qw/	hardware accesslog bios memories slots registry
			controllers monitors ports storages drives inputs
			modems networks printers sounds videos softwares/;
			
	my $device = shift;
	#We keep the actual quality and fidelity
	my $request=$dbh->prepare("SELECT QUALITY,FIDELITY FROM hardware WHERE DEVICEID=".$dbh->quote($device));
	$request->execute;
	my $row = $request->fetchrow_hashref;
	my $quality = $row->{'QUALITY'};
	my $fidelity = $row->{'FIDELITY'};
	$request->finish;
	###
	for (@tables){
		$dbh->do("DELETE FROM $_ WHERE DEVICEID=".$dbh->quote($device));
	}
	$dbh->do("DELETE FROM accountinfo WHERE DEVICEID=".$dbh->quote($DeviceID));
	$dbh->do("UPDATE accountinfo SET DEVICEID=".$dbh->quote($DeviceID)." WHERE DEVICEID=".$dbh->quote($device));
	$dbh->do("UPDATE devices SET DEVICEID=".$dbh->quote($DeviceID)." WHERE DEVICEID=".$dbh->quote($device));
	print LOG localtime().";300;Device $device replaced by $DeviceID;".$ENV{'REMOTE_ADDR'}.";".$r->headers_in->{'User-agent'}.";management\n" if $loglevel;
	$dbh->do("UPDATE hardware SET QUALITY=".$dbh->quote($quality)." WHERE DEVICEID=".$dbh->quote($DeviceID));
	$dbh->do("UPDATE hardware SET FIDELITY=".$dbh->quote($fidelity)." WHERE DEVICEID=".$dbh->quote($DeviceID));
}

1;
